"use strict";

var language = 'en';
fetch('footer_guide.html')
    .then(response => response.text())
    .then(html => {
        let parser = new DOMParser();
        let parsedContent = parser.parseFromString(html, 'text/html');
        document.getElementById('footer-container').appendChild(parsedContent.body);
        loadLanguage();
        window.location.href = "#bluetooth"
    })
    .catch(error => console.error('Error fetching HTML file:', error));

function loadLanguage() {
    language = localStorage.getItem("language");
    if (language == null) {
        language = navigator.language;
        if (language.includes('en')) {
            language = "en";
        }
        localStorage.setItem("language", language);
    }
    getDataLanguage(language)
        .then(data => updateContent(data))
        .catch(error => {
            console.error('Error:', error);
        });
}

async function getDataLanguage(selectedLanguage) {
    try {
        const response = await fetch(`/locales/${selectedLanguage}.json`);
        const data = await response.json();
        return data;
    } catch (error) {
        console.error('Error fetching language data:', error);
        throw error;
    }
}

function updateContent(data) {
    document.querySelectorAll('[data-translate]').forEach(link => {
        const key = link.getAttribute('data-translate');
        link.textContent = data[key];
    });
}

document.body.style["height"] = (screen.height * 0.8) + "px";
