"use strict";

const element_fullscreen = document.getElementById("fullscreen");
const btn_start = document.getElementById("btn-start");
const testGrid = document.getElementById("test-grid");
const title_guide = document.getElementById("guide-touch-screen");
const canvas = document.getElementById("touch-canvas");
const ctx = canvas.getContext("2d");
const ongoingTouches = [];
const rectangles = [];
// Todo: need to change if device has large screen size is big.
var numberCols = 31;
var numberRows = 31;
const numsBlocks = numberCols * numberRows;
var first_update = true;
var countBlocks = 0;
var checked_array = new Array(numsBlocks).fill(0);
var event_touch = "touch";
var start_test = false;
var myTimeout;
const timeout = 60 * 1000 * 5;

function StartTestTouch() {
    myTimeout = window.setTimeout(EndTest, timeout);
    first_update = true;
    ongoingTouches.length = 0;
    rectangles.length = 0;
    checked_array = new Array(numsBlocks).fill(0);
    countBlocks = 0;
    canvas.width = screen.width;
    canvas.height = screen.height;
    canvas.style['display'] = "block";
    btn_start.style['display'] = "none";
    title_guide.style['display'] = "none";
    testGrid.style["display"] = "grid";
    testGrid.style.grid = `repeat(${numberRows},1fr)/repeat(${numberCols},1fr)`;
    testGrid.style.width = screen.width + "px";
    testGrid.style.height = screen.height + "px";
    openFullScreen();
    for (let i = 0; i < numsBlocks; i++) {
        const div = document.createElement('div');
        div.id = `block-${i}`;
        div.classList.add('state-untested');
        testGrid.appendChild(div);
    }
    element_fullscreen.style.touchAction = 'none';
    canvas.addEventListener("pointerdown", handleStart, false);
    canvas.addEventListener("pointerup", handleEnd, false);
    canvas.addEventListener("pointercancel", handleCancel, false);
    canvas.addEventListener("pointermove", handleMove, false);
}

function EndTest() {
    clearTimeout(myTimeout);
    ongoingTouches.length = 0;
    rectangles.length = 0;
    canvas.removeEventListener("pointerdown", handleStart, false);
    canvas.removeEventListener("pointerup", handleEnd, false);
    canvas.removeEventListener("pointercancel", handleCancel, false);
    canvas.removeEventListener("pointermove", handleMove, false);
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    canvas.style['display'] = "none";
    element_fullscreen.style.touchAction = 'auto';
    btn_start.style['display'] = "inline-block";
    title_guide.style['display'] = "block";
    testGrid.style["display"] = "none";
    for (let i = 0; i < numsBlocks; i++) {
        const div = document.getElementById(`block-${i}`);
        testGrid.removeChild(div);
    }
    exitFullScreen();
    if (countBlocks == numsBlocks) {
        alert(status_test["title"] +": "+ status_test["pass"]);
    }
    else {
        alert(status_test["title"] +": "+ status_test["false"]);
    }
}

btn_start.addEventListener("click", StartTestTouch);

function handleStart(evt) {
    if (evt.pointerType === event_touch) {
        ongoingTouches.push(copyTouch(evt));
        ctx.beginPath();
        if(first_update==true)
        {
            for (let i = 0; i < numsBlocks; i++) {
                const div = document.getElementById(`block-${i}`);
                rectangles.push(div.getBoundingClientRect());
            }
            first_update = false;
        }
    }
}
function handleMove(evt) {
    if (evt.pointerType === event_touch) {
        const idx = ongoingTouchIndexById(evt.pointerId);

        if (idx >= 0) {
            ctx.beginPath();
            ctx.moveTo(ongoingTouches[idx].pageX, ongoingTouches[idx].pageY);
            ctx.lineTo(evt.x, evt.y);
            ctx.lineWidth = 2;
            ctx.stroke();
            updateBlock(idx, evt);
            ongoingTouches.splice(idx, 1, copyTouch(evt));
        }
    }
}
function handleEnd(evt) {
    if (evt.pointerType === event_touch) {
        const idx = ongoingTouchIndexById(evt.pointerId);

        if (idx >= 0) {
            ctx.lineWidth = 4;
            ctx.beginPath();
            ctx.moveTo(ongoingTouches[idx].pageX, ongoingTouches[idx].pageY);
            ctx.lineTo(evt.x, evt.y);
            ctx.stroke();
            updateBlock(idx, evt);
            ongoingTouches.splice(idx, 1);
        }
    }
}
function handleCancel(evt) {
    if (evt.pointerType === event_touch) {
        const idx = ongoingTouchIndexById(evt.pointerId);
        ongoingTouches.splice(idx, 1);
    }
}
function copyTouch(touch) {
    return {
        identifier: touch.pointerId,
        pageX: touch.x,
        pageY: touch.y,
    };
}
function ongoingTouchIndexById(idToFind) {
    for (let i = 0; i < ongoingTouches.length; i++) {
        const id = ongoingTouches[i].identifier;

        if (id === idToFind) {
            return i;
        }
    }
    return -1;
}

function updateBlock(idx, evt) {
    const point1 = { x: ongoingTouches[idx].pageX, y: ongoingTouches[idx].pageY };
    const point2 = { x: evt.x, y: evt.y };
    for (let i = 0; i < rectangles.length; i++) {
        if ((checked_array[i] == 0) && checkLineRectIntersection(point1, point2, rectangles[i])) {
            const div = document.getElementById(`block-${i}`);
            div.classList.add("state-tested");
            checked_array[i] = 1;
            countBlocks++;
            if (countBlocks >= numsBlocks) {
                setTimeout(EndTest, 100);
            }
        }
    }
}

function checkLineRectIntersection(point1, point2, rectangle) {
    const x1 = point1.x;
    const y1 = point1.y;
    const x2 = point2.x;
    const y2 = point2.y;

    const rectTopLeftX = rectangle.x;
    const rectTopLeftY = rectangle.y;
    const rectBottomRightX = rectTopLeftX + rectangle.width;
    const rectBottomRightY = rectTopLeftY + rectangle.height;
    const rectMinX = Math.min(rectTopLeftX, rectBottomRightX);
    const rectMaxX = Math.max(rectTopLeftX, rectBottomRightX);
    const rectMinY = Math.min(rectTopLeftY, rectBottomRightY);
    const rectMaxY = Math.max(rectTopLeftY, rectBottomRightY);

    if (x1 < rectMinX && x2 < rectMinX) {
        return false;
    }
    if (x1 > rectMaxX && x2 > rectMaxX) {
        return false;
    }

    if (y1 < rectMinY && y2 < rectMinY) {
        return false;
    }
    if (y1 > rectMaxY && y2 > rectMaxY) {
        return false;
    }

    const lineNormalX = y1 - y2;
    const lineNormalY = x2 - x1;

    const rectCenterX = (rectMinX + rectMaxX) / 2;
    const rectCenterY = (rectMinY + rectMaxY) / 2;

    const rectHalfWidth = (rectMaxX - rectMinX) / 2;
    const rectHalfHeight = (rectMaxY - rectMinY) / 2;

    const separation = Math.abs((x2 - rectCenterX) * lineNormalX + (y2 - rectCenterY) * lineNormalY);

    const projectedHalfWidth = rectHalfWidth * Math.abs(lineNormalX) + rectHalfHeight * Math.abs(lineNormalY);

    return separation <= projectedHalfWidth;
}